/*
 *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
 *	<manutm007@gmail.com>
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "tickr.h"

#ifdef G_OS_WIN32
FILE			*stdout_fp, *stderr_fp;
#endif
static TickerEnv	*env;
static Resource	*resrc;
static FList		*feed_list;
static FList		*feed_selection;
static Params		*prm;
static int		instance_id = 0;
static gboolean	no_ui = FALSE;
static int		shift_counter = 0;
static GtkWidget	*popup_menu;
static GtkAccelGroup	*popup_menu_accel_group;
static int		popup_menu_accel_group_attached;
static GtkWidget	*main_hbox, *vbox_ticker, *vbox_clock;
static int		speed_up_flag = FALSE;
static int		speed_down_flag = FALSE;

#define START_PAUSE_TICKER_WHILE_OPENING\
	int	suspend_rq_bak = env->suspend_rq;\
	env->suspend_rq = TRUE;

#define END_PAUSE_TICKER_WHILE_OPENING\
	env->suspend_rq = suspend_rq_bak;

/* Sometimes, we need to temporarily re-enable popups, just to get error messages. */
#define START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING\
	int	suspend_rq_bak;\
	char	disable_popups_bak;\
	suspend_rq_bak = env->suspend_rq;\
	env->suspend_rq = TRUE;\
	disable_popups_bak = get_params()->disable_popups;\
	get_params()->disable_popups = 'n';

#define END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING\
	get_params()->disable_popups = disable_popups_bak;\
	env->suspend_rq = suspend_rq_bak;

/* IF_UI_ALLOWED now replace USE_GUI compile macro which has been there
 * and ***unused*** for ages and breaks compile. Plus it's a runtime variable */
#define IF_UI_ALLOWED	if (!no_ui)

/* Prototypes for a few static funcs */
static void show_resource_info();
static gint shift2left_callback();
static void check_time_load_resource(check_time_mode);

TickerEnv *get_ticker_env()
{
	return env;
}

Resource *get_resource()
{
	return resrc;
}

FList *get_feed_list()
{
	return feed_list;
}

void set_feed_list(FList *list)
{
	feed_list = list;
}

FList *get_feed_selection()
{
	return feed_selection;
}

void set_feed_selection(FList *list)
{
	feed_selection = list;
}

Params *get_params()
{
	return prm;
}

int get_instance_id()
{
	return instance_id;
}

void check_main_win_always_on_top()
{
	if (prm->always_on_top == 'y')
		gtk_window_set_keep_above(GTK_WINDOW(env->win), TRUE);
	else
		gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);
}

/*
 * funct_name2(***NO args***) only used to call funct_name(***args***)
 */
static void get_new_url2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	IF_UI_ALLOWED
		get_new_url(resrc);
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void open_new_txt_file2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	IF_UI_ALLOWED
		open_new_txt_file(resrc);
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void import_opml_file2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	IF_UI_ALLOWED
		import_opml_file();
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void export_opml_file2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	IF_UI_ALLOWED
		export_opml_file();
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void show_resource_info2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	show_resource_info();
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

void modify_params2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	IF_UI_ALLOWED
		if (modify_params(prm) == 1)	/* 1 means we want to immediately reopen the dialog */
			modify_params2(prm);
	END_PAUSE_TICKER_WHILE_OPENING
}

static void connection_settings2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	IF_UI_ALLOWED
		if (connection_settings(AUTH_PAGE) == GTK_RESPONSE_OK)
			current_feed();
	END_PAUSE_TICKER_WHILE_OPENING
}

static void import_params2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	IF_UI_ALLOWED
		import_params();
	END_PAUSE_TICKER_WHILE_OPENING
}

static void export_params2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	IF_UI_ALLOWED
		export_params();
	END_PAUSE_TICKER_WHILE_OPENING
}

static void pause_on_mouseover_enabled_warning_once()
{
	static int	first_run = -1;

	if (prm->pause_on_mouseover == 'y') {
		if (first_run == -1)
			warning(FALSE, 1,
				"Setting 'Pause ticker on mouse-over' is enabled. When so, playing/pausing\n"
				"is always controlled by mouse pointer motions as well.");
		first_run++;
		first_run &= 1;
	}
}

static void ticker_play()
{
	env->suspend_rq = FALSE;
	pause_on_mouseover_enabled_warning_once();
}

static void ticker_pause()
{
	env->suspend_rq = TRUE;
	pause_on_mouseover_enabled_warning_once();
}

static void ticker_reload()
{
	current_feed();
	env->reload_rq = TRUE;
	env->suspend_rq = FALSE;
}

static void first_feed2()
{
	if (env->selection_mode == MULTIPLE)
		first_feed();
	else
		info_win_wait("Single selection mode", INFO_WIN_WAIT_TIMEOUT);
}

static void previous_feed2()
{
	if (env->selection_mode == MULTIPLE)
		previous_feed();
	else
		info_win_wait("Single selection mode", INFO_WIN_WAIT_TIMEOUT);
}

static void next_feed2()
{
	if (env->selection_mode == MULTIPLE)
		next_feed();
	else
		info_win_wait("Single selection mode", INFO_WIN_WAIT_TIMEOUT);
}

static void last_feed2()
{
	if (env->selection_mode == MULTIPLE)
		last_feed();
	else
		info_win_wait("Single selection mode", INFO_WIN_WAIT_TIMEOUT);
}

void toggle_speed_up_flag()
{
	speed_up_flag = TRUE;
}

void toggle_speed_down_flag()
{
	speed_down_flag = TRUE;
}

static void help_win2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	help_win();
	END_PAUSE_TICKER_WHILE_OPENING
}

static void online_help2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	online_help();
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void check_for_updates2()
{
	START_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
	check_for_updates();
	END_PAUSE_TICKER_ENABLE_POPUPS_WHILE_OPENING
}

static void about_win2()
{
	START_PAUSE_TICKER_WHILE_OPENING
	about_win();
	END_PAUSE_TICKER_WHILE_OPENING
}

static void show_resource_info()
{
	GtkWidget	*dialog, *table, *label[7], *close_but;
	char		tmp1[256], *tmp2, *tmp3;
	int		i, j;

	if (resrc->type != RESRC_URL && resrc->type != RESRC_FILE) {
		info_win("Resource Properties", "\nNo information available\n", INFO_ERROR, FALSE);
		return;
	}

	gtk_window_set_keep_above(GTK_WINDOW(get_ticker_env()->win), FALSE);

	dialog = gtk_dialog_new_with_buttons(
			"Resource Properties", GTK_WINDOW(get_ticker_env()->win),
			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
			NULL);

	close_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL_CLOSE);
	close_but = close_but;	/* To get rid of one stupid compiler warning */

	set_tickr_icon_to_dialog(GTK_WINDOW(dialog));
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);

	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(esc_key_pressed), NULL);
	g_signal_connect(G_OBJECT(dialog), "delete_event", G_CALLBACK(force_quit_dialog), NULL);

	table = gtk_table_new(3, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(table), 15);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	label[0] = gtk_label_new("        ");
	if (resrc->type == RESRC_URL) {
		label[1] = gtk_label_new("Resource type:");
		snprintf(tmp1, 256, "%s Feed", resrc->format == RSS_2_0 ? "RSS 2.0" : \
			(resrc->format == RSS_ATOM ? "Atom" : "RSS 1.0"));
		label[2] = gtk_label_new(tmp1);

		label[3] = gtk_label_new("Feed title:");
		label[4] = gtk_label_new(resrc->feed_title);

		label[5] = gtk_label_new("Feed URL:");
		tmp2 = l_str_new("<a href=\"");
		tmp3 = malloc2(sizeof(char) * FILE_NAME_MAXLEN * 2);
		/* Need to replace all "&" with "&amp;" to use markup. */
		for (i = 0, j = 0; resrc->id[i] != '\0' && j < (FILE_NAME_MAXLEN * 2) - 4; i++, j++)
			if (resrc->id[i] != '&')
				tmp3[j] = resrc->id[i];
			else {
				str_n_cpy(tmp3 + j, "&amp;", 5);
				j += 4;
			}
		tmp3[j] = '\0';
		tmp2 = l_str_cat(tmp2, tmp3);
		tmp2 = l_str_cat(tmp2, "\">");
		tmp2 = l_str_cat(tmp2, tmp3);
		tmp2 = l_str_cat(tmp2, "</a>");
		label[6] = gtk_label_new(tmp2);
		l_str_free(tmp2);
		free2(tmp3);
		gtk_label_set_use_markup(GTK_LABEL(label[6]), TRUE);

		for (i = 0; i < 7; i++)
			gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	} else if (resrc->type == RESRC_FILE) {
		label[1] = gtk_label_new("Resource type:  ");
		label[2] = gtk_label_new("File");

		label[3] = gtk_label_new("File name:  ");
		label[4] = gtk_label_new(resrc->id);
		gtk_label_set_selectable(GTK_LABEL(label[4]), TRUE);

		for (i = 0; i < 5; i++)
			gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
	}

	gtk_table_attach_defaults(GTK_TABLE(table), label[0], 1, 2, 0, 1);

	gtk_table_attach_defaults(GTK_TABLE(table), label[1], 0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), label[2], 2, 3, 0, 1);

	gtk_table_attach_defaults(GTK_TABLE(table), label[3], 0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), label[4], 2, 3, 1, 2);

	if (resrc->type == RESRC_URL) {
		gtk_table_attach_defaults(GTK_TABLE(table), label[5], 0, 1, 2, 3);
		gtk_table_attach_defaults(GTK_TABLE(table), label[6], 2, 3, 2, 3);
	}

	gtk_widget_show_all(dialog);
	while (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_CANCEL_CLOSE);

	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
}

#ifdef G_OS_WIN32
static int easily_link_with_browser2()
{
	char		*browser_cmd_p, browser_cmd[512], tmp[1024];
	unsigned int	i, count, start = 0, end = 0;
	int		exit_status = NO;

	if ((browser_cmd_p = (char *)get_default_browser_from_win32registry()) != NULL) {
		/* Find 1st str in browser_cmd */
		str_n_cpy(tmp, browser_cmd_p, 511);
		for (i = 0, count = 0; i < strlen(tmp); i++) {
			if (tmp[i] == '"') {
				count++;
				if (count == 1)
					start = i + 1;
				else if (count == 2)
					end = i - 1;
			}
		}
		str_n_cpy(browser_cmd, tmp + start, MIN((end - start + 1), 511));
		snprintf(tmp, 1024,
			"\nThis is the Shell command that opens your default Browser:\n\n%s\n\n"
			"Do you want to use it (recommended) ?\n", browser_cmd);
		if (question_win(tmp) == YES) {
			exit_status = YES;
			str_n_cpy(prm->open_link_cmd, browser_cmd, FILE_NAME_MAXLEN);
			str_n_cpy(prm->open_link_args, "", FILE_NAME_MAXLEN);
			if (question_win("Save this parameter ?") == YES)
				save_to_config_file(prm);
		} else
			info_win("", "\nCancelled.\n"
				"You may too set this parameter manually in the Preferences window.\n",
				INFO, FALSE);
	} else
		info_win("", "\nCan't find Browser shell command."
			"You will have to set this parameter manually in the Preferences window.\n",
			INFO_ERROR, FALSE);
	return exit_status;
}

static void easily_link_with_browser()
{
	gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);
	easily_link_with_browser2();
	check_main_win_always_on_top();
}
#endif

/* If link found, copy it into env->active_link and return rank (starting at 1),
 * otherwise, return -1 (with link = "") */
static int get_visible_link_and_rank(int position_x_in_drwa)
{
	int	i, location_on_surface;

	if (position_x_in_drwa > -1) {
		location_on_surface = shift_counter * prm->shift_size + position_x_in_drwa;
		for (i = 0; i < NFEEDLINKANDOFFSETMAX; i++) {
			if (resrc->link_and_offset[i].offset_in_surface > location_on_surface) {
				str_n_cpy(env->active_link, resrc->link_and_offset[i].url, FILE_NAME_MAXLEN);
				return i;
			}
		}
	}
	env->active_link[0] = '\0';
	return -1;
}

static void open_link()
{
	char		tmp1[FILE_NAME_MAXLEN + 1], tmp2[FILE_NAME_MAXLEN + 1];
/*#ifdef G_OS_WIN32
	char		*tmp;
#endif*/
	char		*argv[32];	/* Up to 32 - 3 (prog name, url, NULL) args */
	GPid		pid;
	GError		*error = NULL;
	int		i, j;

	if (prm->open_link_cmd[0] == '\0') {
#ifndef G_OS_WIN32
		/* TODO: should look for default browser */
		warning(FALSE, 2, "Can't launch Browser: no command is defined.\n",
			"Please set the 'Open in Browser' option in the Preferences window.");
		return;
#else
		easily_link_with_browser();
		if (prm->open_link_cmd[0] == '\0') {
			warning(FALSE, 2, "Can't launch Browser: no command is defined.\n",
				"Please set the 'Open in Browser' option in the Preferences window.");
			return;
		}
#endif
	}

	if (env->active_link[0] == '\0') {
		warning(FALSE, 1, "No link found");
		return;
	}

	fprintf(STD_OUT, "Spawning: %s %s %s\n", prm->open_link_cmd, prm->open_link_args, env->active_link);
	str_n_cpy(tmp1, prm->open_link_cmd, FILE_NAME_MAXLEN);
	str_n_cpy(tmp2, prm->open_link_args, FILE_NAME_MAXLEN);
	argv[0] = tmp1;
	for (i = 0, j = 1; tmp2[i] != '\0' && j < 32 + 1 - 3; j++) {
		argv[j] = &tmp2[i];
		while (tmp2[++i] != ' ')
			if (tmp2[i] == '\0')
				break;
		tmp2[i] = '\0';
		while (tmp2[++i] == ' ');
	}
	argv[j] = env->active_link;
	argv[j + 1] = NULL;
	if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &pid, &error)) {
		warning(FALSE, 3, APP_NAME": Can't create process ", argv[0], " - ", error->message);
#ifndef G_OS_WIN32
		info_win("", "Please check the 'Open in Browser' option in the Preferences window.",
			INFO_WARNING, FALSE);
#else
		if (easily_link_with_browser2() == YES)
			open_link();
#endif
	} else
		g_spawn_close_pid(pid);
}

static void open_link2()
{
	get_visible_link_and_rank(env->drwa_width / 2);
	open_link();
}

/* Continuously get mouse x position inside drawing area.
 * Do nothing if outside, so must be used only by mouse-over-drawing-area /
 * click-on-drawing-area funcs. */
static gint track_mouse_position_x(GtkWidget *widget, GdkEventMotion *event)
{
	widget = widget;
	if (event->type == GDK_MOTION_NOTIFY)	/* Means mouse has moved inside widget  */
		env->mouse_x_in_drwa = (int)(event->x);
	return FALSE;
}

static gint left_click_on_drawing_area(GtkWidget *widget, GdkEventButton *event_but)
{
	widget = widget;
	if (prm->disable_leftclick != 'y' && event_but->type == GDK_BUTTON_PRESS &&\
			event_but->button == 1) {	/* 1 = mouse left button */
		get_visible_link_and_rank(env->mouse_x_in_drwa);
		open_link();
		return TRUE;
	} else
		return FALSE;
}

static gint right_click_on_drawing_area(GtkWidget *widget, GdkEventButton *event_but)
{
	widget = widget;
	if (event_but->type == GDK_BUTTON_PRESS && event_but->button == 3) {	/* 3 = mouse right button */
		/* Popup main menu */
		gtk_menu_popup(GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, event_but->button, event_but->time);
		return TRUE;
	} else
		return FALSE;
}

static const char *get_win_title_text()
{
	static char	tmp1[2][256 + 64], tmp2[2][64];
	static int	i = -1;

	/* To allow 2 simultaneous calls */
	i++;
	i &= 1;

	if (resrc->type == RESRC_URL)
		str_n_cpy(tmp1[i], resrc->feed_title, 256);
	else if (resrc->type == RESRC_FILE)
		str_n_cpy(tmp1[i], resrc->id, 256);
	else
		str_n_cpy(tmp1[i], "No resource", 256);
	snprintf(tmp2[i], 64, "  |  %s-%s", APP_NAME, APP_VERSION_NUMBER);
	/* which is better?
	snprintf(tmp2[i], 64, "  |  %s", APP_NAME);*/
	return (const char *)str_n_cat(tmp1[i], tmp2[i], 63);
}

char *get_feed_extra_info(int rank)
{
	FILE		*fp;
	char		*str1, tmp[4];
	size_t		str1_size = 8 * 1024;
	static char	*str2 = NULL;
	int		tag_found = FALSE;

	if (str2 != NULL)
		l_str_free(str2);
	str2 = l_str_new("");
	str2[0] = '\0';
	if (rank == -1)
		return str2;
	if ((fp = g_fopen(resrc->xml_dump_extra, "rb")) != NULL) {
		str1 = malloc2(str1_size * sizeof(char));
#ifndef G_OS_WIN32
		while (getline(&str1, &str1_size, fp) != -1) {
#else
		while (fgets(str1, str1_size, fp) != NULL) {
#endif
			if (str1[0] == ITEM_TITLE_TAG_CHAR || str1[0] == ITEM_DES_TAG_CHAR) {
				if (tag_found)
					break;
				if (rank == atoi(str_n_cpy(tmp, str1 + 1, 3))) {
					tag_found = TRUE;
					str2 = l_str_cat(str2, str1 + 4);
				}
			} else if (tag_found)
				str2 = l_str_cat(str2, str1);
		}
		free2(str1);
		fclose(fp);
	}
	remove_trailing_whitespaces_from_str(str2);
	return str2;
}

static const char *get_ticker_tooltip_text(int rank)
{
	static char	*tooltip_text = NULL;
	/*gboolean	feed_extra_info = FALSE;*/

	if (tooltip_text != NULL) {
		l_str_free(tooltip_text);
		tooltip_text = NULL;
	}
	if ((prm->item_title == 'y' && prm->item_description == 'n') ||\
			(prm->item_title == 'n' && prm->item_description == 'y')) {
		tooltip_text = l_str_new(get_feed_extra_info(rank));
		if (tooltip_text[0] != '\0')
			/*feed_extra_info = TRUE;
		if (feed_extra_info)*/
			tooltip_text = l_str_cat(tooltip_text, "\n\n");
			/* which is better?
			tooltip_text = l_str_cat(tooltip_text, "\n\n[ ");*/
		tooltip_text = l_str_cat(tooltip_text, get_win_title_text());
		/*if (feed_extra_info)
			tooltip_text = l_str_cat(tooltip_text, " ]");*/
		return tooltip_text;
	} else
		return get_win_title_text();
}

/* Pause ticker when mouse pointer is over (opened) popup menu. */
static gint mouse_over_opened_popup_menu(GtkWidget *widget, GdkEvent *event)
{
	widget = widget;
	if (prm->pause_on_mouseover == 'y') {
		if (event->type == GDK_EXPOSE)
			env->suspend_rq = TRUE;
		else if (event->type == GDK_LEAVE_NOTIFY)
			env->suspend_rq = FALSE;
	}
	return FALSE;
}

/* Pause ticker on mouseover. */
static gint mouse_over_drawing_area(GtkWidget *widget, GdkEvent *event)
{
	int	rank;

	widget = widget;
	if (event->type == GDK_ENTER_NOTIFY || event->type == GDK_LEAVE_NOTIFY) {
		if (prm->pause_on_mouseover == 'y') {
			if (event->type == GDK_ENTER_NOTIFY)
				env->suspend_rq = TRUE;
			else /* USELESS: if (event->type == GDK_LEAVE_NOTIFY)*/
				env->suspend_rq = FALSE;
		}
		if (event->type == GDK_ENTER_NOTIFY) {
			while (gdk_events_pending())	/* So that "motion-notify-event" is processed now */
				gtk_main_iteration();
			rank = get_visible_link_and_rank(env->mouse_x_in_drwa);
			gtk_widget_set_tooltip_text(widget, get_ticker_tooltip_text(rank));
			gtk_tooltip_trigger_tooltip_query(gdk_display_get_default());
			return TRUE;
		}
	}
	return FALSE;
}

static gint mouse_wheel_scroll_on_drawing_area(GtkWidget *widget, GdkEvent *event)
{
	char mouse_wheel_scroll;

	widget = widget;
	if (event->type == GDK_SCROLL) {
		mouse_wheel_scroll = prm->mouse_wheel_scroll;
		if (event->scroll.state & GDK_CONTROL_MASK) {
			/* If ctrl pressed, invert mouse_wheel_scroll values */
			if (mouse_wheel_scroll == 's')
				mouse_wheel_scroll = 'f';
			else if (mouse_wheel_scroll == 'f')
				mouse_wheel_scroll = 's';
		}
		if (mouse_wheel_scroll == 's') {
			if (event->scroll.direction == GDK_SCROLL_UP)
				toggle_speed_up_flag();
			else if (event->scroll.direction == GDK_SCROLL_DOWN)
				toggle_speed_down_flag();
			env->suspend_rq = FALSE;
			return TRUE;
		} else if (mouse_wheel_scroll == 'f') {
			if (env->selection_mode == MULTIPLE) {
				if (event->scroll.direction == GDK_SCROLL_UP)
					quick_feed_picker(GDK_SCROLL_UP);
				else if (event->scroll.direction == GDK_SCROLL_DOWN)
					quick_feed_picker(GDK_SCROLL_DOWN);
				env->suspend_rq = FALSE;
			} else
				warning(FALSE, 2, "Single selection mode\n",
					"(You have set 'Mouse wheel scrolling applies to: Feed')");
			return TRUE;
		}
	}
	return FALSE;
}

/*
 * Popup menu stuff
 */
static GtkItemFactoryEntry popup_menu_item[] = {
	{"/_File",		NULL, NULL, 0, "<Branch>", NULL},

	{"/File/Open Feed (_RSS|Atom)", "<control>R", get_new_url2, 0, "<StockItem>",
				(gconstpointer)RSS_ICON},

	{"/File/Open _Text File", "<control>T", open_new_txt_file2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_OPEN},

	{"/File/sep",		NULL, NULL, 0, "<Separator>", NULL},

	{"/File/_Import Feed List (OPML)", "<control>I", import_opml_file2, 0, NULL, NULL},

	{"/File/_Export Feed List (OPML)", "<control>E", export_opml_file2, 0, NULL, NULL},

	{"/File/sep",		NULL, NULL, 0, "<Separator>", NULL},

	{"/File/Resource _Properties", "<control>P", show_resource_info2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_PROPERTIES},

	{"/File/sep",		NULL, NULL, 0, "<Separator>", NULL},

	{"/File/_Quit",		"<control>Q", gtk_main_quit, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_QUIT},

	{"/_Edit",		NULL, NULL, 0, "<Branch>", NULL},

	{"/Edit/Preference_s",	"<control>S", modify_params2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_PREFERENCES},

	{"/Edit/Connection Settings", "<control>", connection_settings2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_CONNECT},

	{"/Edit/sep",		NULL, NULL, 0, "<Separator>", NULL},

	{"/Edit/Import Preferences", "<control>", import_params2, 0, NULL, NULL},

	{"/Edit/Export Preferences", "<control>", export_params2, 0, NULL, NULL},

	{"/_Control",		NULL, NULL, 0, "<Branch>", NULL},

	{"/Control/Open Link in _Browser", "<control>B", open_link2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_JUMP_TO/*GTK_STOCK_REDO*/},

	{"/Control/sep",	NULL, NULL, 0, "<Separator>", NULL},

	{"/Control/Play",	"<control>J", ticker_play, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_MEDIA_PLAY},

	{"/Control/Pause",	"<control>K", ticker_pause, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_MEDIA_PAUSE},

	{"/Control/Reload",	"<control>L", ticker_reload, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_REFRESH},

	{"/Control/sep",	NULL, NULL, 0, "<Separator>", NULL},

	{"/Control/First Feed",	"<control>", first_feed2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_GOTO_FIRST},

	{"/Control/Previous Feed", "<control>", previous_feed2, 0,  "<StockItem>",
				(gconstpointer)GTK_STOCK_GO_BACK},

	{"/Control/Next Feed",	"<control>", next_feed2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_GO_FORWARD},

	{"/Control/Last Feed",	"<control>", last_feed2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_GOTO_LAST},

	{"/Control/sep",	NULL, NULL, 0, "<Separator>", NULL},

	{"/Control/Speed Up",	"<control>U", toggle_speed_up_flag, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_GO_UP},

	{"/Control/Speed Down",	"<control>D", toggle_speed_down_flag, 0,  "<StockItem>",
				(gconstpointer)GTK_STOCK_GO_DOWN},

	{"/_Help",		NULL, NULL, 0, "<Branch>", NULL},

	{"/Help/Quick Help",	"F1", help_win2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_HELP},

	{"/Help/Online Help",	"<control>H", online_help2, 0, NULL, NULL},

	{"/Help/Check for Updates", "<control>", check_for_updates2, 0, NULL, NULL},

	{"/Help/sep",		NULL, NULL, 0, "<Separator>", NULL},

	{"/Help/_About",	"<control>A", about_win2, 0, "<StockItem>",
				(gconstpointer)GTK_STOCK_ABOUT},
};

static gint n_popup_menu_items = sizeof(popup_menu_item) / sizeof(popup_menu_item[0]);

static GtkWidget *get_popup_menu_menu(GtkWidget *win)
{
	GtkItemFactory	*item_factory;

	win = win;
	popup_menu_accel_group = gtk_accel_group_new();
	item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", popup_menu_accel_group);
	gtk_item_factory_create_items(item_factory, n_popup_menu_items, popup_menu_item, NULL);
	return gtk_item_factory_get_widget(item_factory, "<main>");
}

static gint update_win_dims()
{
	gtk_widget_set_size_request(env->drw_a, env->drwa_width, env->drwa_height);
	gtk_widget_set_size_request(env->drwa_clock, env->drwa_clock_width, env->drwa_height);

	gtk_widget_set_size_request(env->win, 0, 0);
	gtk_window_resize(GTK_WINDOW(env->win), env->drwa_width + env->drwa_clock_width, env->drwa_height);

	gtk_window_unmaximize(GTK_WINDOW(env->win));
	return TRUE;
}

/* Not used anymore
gboolean params_have_been_changed(Params *new_prm, int n_required_runs)
{
	static Params	prm_bak;
	static int	initial_runs = 0;

	if (initial_runs < n_required_runs) {
		// 'fake params changes' are required at program startup
		initial_runs++;
		memcpy((void *)&prm_bak, (const void *)new_prm, sizeof(Params));
		return TRUE;
	} else if (memcmp((const void *)new_prm, (const void *)&prm_bak, sizeof(Params)) != 0) {
		memcpy((void *)&prm_bak, (const void *)new_prm, sizeof(Params));
		return TRUE;
	} else
		return FALSE;
}*/

gboolean win_params_have_been_changed(Params *new_prm, int n_required_runs)
{
	static Params	prm_bak0;
	Params		*prm_bak = &prm_bak0;
	static int	initial_runs = 0;
	gboolean	changes = FALSE;

	if (initial_runs < n_required_runs) {
		/* 'fake params changes' are required at program startup */
		initial_runs++;
		memcpy((void *)prm_bak, (const void *)new_prm, sizeof(Params));
		return TRUE;
	} else if (strcmp(new_prm->font_n_s, prm_bak->font_n_s) != 0)
		changes = TRUE;
	else if (new_prm->win_x != prm_bak->win_x)
		changes = TRUE;
	else if (new_prm->win_y != prm_bak->win_y)
		changes = TRUE;
	else if (new_prm->win_w != prm_bak->win_w)
		changes = TRUE;
	else if (new_prm->win_h != prm_bak->win_h)
		changes = TRUE;
	else if (new_prm->windec != prm_bak->windec)
		changes = TRUE;
	else if (new_prm->always_on_top != prm_bak->always_on_top)
		changes = TRUE;
	else if (new_prm->icon_in_taskbar != prm_bak->icon_in_taskbar)
		changes = TRUE;
	else if (new_prm->win_sticky != prm_bak->win_sticky)
		changes = TRUE;
	else if (new_prm->clock != prm_bak->clock)
		changes = TRUE;
	else if (strcmp(new_prm->clock_font_n_s, prm_bak->clock_font_n_s) != 0)
		changes = TRUE;
	if (changes == TRUE) {
		memcpy((void *)prm_bak, (const void *)new_prm, sizeof(Params));
		return TRUE;
	} else
		return FALSE;
}

/*
 * Create new or next cairo image surface and compute stuff to redraw window.
 */
static int compute_surface_and_win()
{
	char 	fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
	char	cfname[FONT_NAME_MAXLEN + 1], cfsize[FONT_SIZE_MAXLEN + 1];
	gint	height1, height2;
	int	size, render_exit_status, i;

	/*
	 * Compute font size from requested height if > 0
	 */
	split_font(prm->font_n_s, fname, fsize);
	if (prm->win_h > 0)
		snprintf(fsize, FONT_SIZE_MAXLEN + 1, "%3d",
			get_fsize_from_layout_height((gint)prm->win_h, fname));
	/* In all cases, font size can't be > FONT_MAXSIZE */
	if (atoi(fsize) > FONT_MAXSIZE)
		snprintf(fsize, FONT_SIZE_MAXLEN + 1, "%3d", FONT_MAXSIZE);
	compact_font(prm->font_n_s, fname, fsize);
	/*
	 * Compute clock font size
	 * clock height = ticker height unless clock font size is set
	 * in which case clock height is always <= ticker height
	 */
	split_font(prm->clock_font_n_s, cfname, cfsize);
	if (strcmp(fname, cfname) == 0) {
		if (atoi(cfsize) > atoi(fsize))
			str_n_cpy(cfsize, fsize, FONT_SIZE_MAXLEN);
	} else {
		height1 = get_layout_height_from_fnamesize(prm->clock_font_n_s);
		height2 = get_layout_height_from_fnamesize(prm->font_n_s);
		if (height1 > height2) {
			size = get_fsize_from_layout_height(height2, cfname);
			snprintf(cfsize, FONT_SIZE_MAXLEN + 1, "%3d", size);
		}
	}
	compact_font(prm->clock_font_n_s, cfname, cfsize);
	/*
	 * Compute ticker width
	 */
	if (prm->win_w >= DRWA_WIDTH_MIN && prm->win_w <= env->screen_w)
		env->drwa_width = prm->win_w;
	else if (prm->win_w < DRWA_WIDTH_MIN)
		env->drwa_width = DRWA_WIDTH_MIN;
	else if (prm->win_w > env->screen_w)
		env->drwa_width = env->screen_w;
	env->drwa_width -= (env->drwa_clock_width = get_clock_width(prm));
	/*
	 * Reset link_and_offset stuff
	 */
	if(resrc->id[0] == '\0' && resrc->fp != NULL) {
		fclose(resrc->fp);
		resrc->fp = NULL;
		for (i = 0; i < NFEEDLINKANDOFFSETMAX; i++) {
			resrc->link_and_offset[i].url[0] = '\0';
			resrc->link_and_offset[i].offset_in_surface = 0;
		}
		if(resrc->fp_extra != NULL) {
			fclose(resrc->fp_extra);
			resrc->fp_extra = NULL;
		}
	}
	/*
	 * Create cairo image surface of rendered text (one long single line)
	 */
	env->c_surf = render_stream_to_surface(resrc->fp, resrc->link_and_offset, prm, &render_exit_status);
	if (render_exit_status != OK && render_exit_status != RENDER_NO_RESOURCE) {
		switch(render_exit_status) {
			case RENDER_NULL_ENDINGSTR:
				warning(FALSE, 1, "render_stream_to_surface(): ending string = NULL");
				break;
			case RENDER_GETLINE_ERROR:
				warning(FALSE, 1, "render_stream_to_surface(): getline() error");
				break;
			case RENDER_LINE_TOO_LONG:
				warning(FALSE, 1, "render_stream_to_surface(): line too long\n");
				break;
			case RENDER_CAIRO_IMAGE_SURFACE_TOO_WIDE:
				big_error(RENDER_CAIRO_IMAGE_SURFACE_TOO_WIDE, 1, "render_stream_to_surface(): "
					"cairo image surface too wide (> 32 K pixels)");
				break;
			case RENDER_CREATE_CAIRO_IMAGE_SURFACE_ERROR:
				big_error(RENDER_CREATE_CAIRO_IMAGE_SURFACE_ERROR, 1, "render_stream_to_surface(): "
					"can't create cairo image surface");
				break;
			default: warning(FALSE, 2, "render_stream_to_surface(): unknown error ",
					itoa2(render_exit_status));
				break;
		}
	}
	if (env->c_surf == NULL)
		big_error(render_exit_status, 1,
			"render_stream_to_surface(): cairo image surface = NULL");
	env->surf_width = cairo_image_surface_get_width(env->c_surf);
	env->surf_height = cairo_image_surface_get_height(env->c_surf);
	env->drwa_height = (MIN(env->surf_height, env->screen_h));
	/* Win title */
	gtk_window_set_title(GTK_WINDOW(env->win), get_win_title_text());
	/* Whole window transparency */
	gtk_window_set_opacity(GTK_WINDOW(env->win), prm->win_transparency);
	/* Following code needs to be run
	* - twice at program startup (ie once after gtk_widget_show_all() has been called)
	* - whenever some window params have been changed
	* - not everytime a new feed is loaded. */
	if (win_params_have_been_changed(prm, 2)) {
		gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);
		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(env->win), FALSE);
		gtk_window_unstick(GTK_WINDOW(env->win));
		/* Win decoration */
		if (prm->windec == 'y') {
			gtk_window_set_decorated(GTK_WINDOW(env->win), TRUE);
			if (!popup_menu_accel_group_attached) {
				gtk_window_add_accel_group(GTK_WINDOW(env->win), popup_menu_accel_group);
				popup_menu_accel_group_attached = TRUE;
			}
		} else if (prm->windec == 'n') {
			gtk_window_set_decorated(GTK_WINDOW(env->win), FALSE);
			if (popup_menu_accel_group_attached) {
				gtk_window_remove_accel_group(GTK_WINDOW(env->win), popup_menu_accel_group);
				popup_menu_accel_group_attached = FALSE;
			}
		}
		/* Clock */
		if (prm->clock == 'l')
			gtk_box_reorder_child(GTK_BOX(main_hbox), vbox_clock, 0);
		else if (prm->clock == 'r')
			gtk_box_reorder_child(GTK_BOX(main_hbox), vbox_clock, 1);
		/* Move win */
		gtk_window_move(GTK_WINDOW(env->win), prm->win_x, prm->win_y);
		/* Window icon in taskbar? */
		if (prm->icon_in_taskbar == 'y')
			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(env->win), FALSE);
		else if (prm->icon_in_taskbar == 'n')
			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(env->win), TRUE);
		/* Window on all desktops? */
		if (prm->win_sticky == 'y')
			gtk_window_stick(GTK_WINDOW(env->win));
		else if (prm->win_sticky == 'n')
			gtk_window_unstick(GTK_WINDOW(env->win));
		check_main_win_always_on_top();
		gtk_window_present(GTK_WINDOW(env->win));
	}
	while (gtk_events_pending())
		gtk_main_iteration();
	return render_exit_status;
}

/*
 * Timeout handler to render one part of cairo image surface onto drawing
 * area - image is shifted to left by <shift_size> pixels
 *
 * Handler 'listens' to flags: suspend_rq, reload_rq, compute_rq and
 * stream_fully_read
 *
 * Notes:
 * - reload_rq may be a misleading name as, in multiple selection mode,
 * we will load next stream, not reload the same one
 * - stream_fully_read is only relevant in multiple selection mode because
 * there is no need to reload each time in single selection mode
 */
static gint shift2left_callback()
{
	cairo_t		*cr;
	GdkRectangle	r;
	char		tmp[256];
	int		i;

	if (env->suspend_rq)
		return TRUE;	/* Does nothing - just return */
	/* Can't remember where this magical -8 little adjustment comes from... */
	else if ((shift_counter * prm->shift_size < env->surf_width - env->drwa_width - 8) &&\
			!env->reload_rq && !env->compute_rq) {
		env->suspend_rq = TRUE;
		/*
		 * Draw onto ticker area
		 * (we now use cairo instead of deprecated gdk_draw_ stuff)
		 */
		/* Checking first can't hurt */
		if (env->c_surf == NULL)
			big_error(SHIFT2LEFT_NULL_CAIRO_IMAGE_SURFACE, 1,
				"shift2left_callback(): cairo image surface = NULL");
		/* Double buffering disabled for drawing area so we need this */
		gdk_window_get_geometry(env->drw_a->window, &(r.x), &(r.y), NULL, NULL, NULL);
		if (prm->clock == 'l')			/* We must do that but not quite sure why */
			r.x -= env->drwa_clock_width;
		r.width = env->drwa_width;
		r.height = env->drwa_height;
		gdk_window_begin_paint_rect(env->drw_a->window, &r);
		/* Cairo stuff */
		cr = gdk_cairo_create(GDK_DRAWABLE(env->drw_a->window));
		/* dest_x - src_x, dest_y - src_y */
		cairo_set_source_surface(cr, env->c_surf, - (shift_counter++ * prm->shift_size), 0);
		/* dest_x, dest_y */
		cairo_rectangle(cr, 0, 0, env->drwa_width, env->drwa_height);
		cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
		cairo_fill(cr);
		cairo_destroy(cr);
		/* We're done */
		gdk_window_end_paint(env->drw_a->window);
		env->suspend_rq = FALSE;
		/*
		 * Speeding up/down on the fly
		 */
		if (speed_up_flag == TRUE || speed_down_flag == TRUE) {
			if (speed_up_flag == TRUE) {
				speed_up_flag = FALSE;
				if (prm->delay > 1)
					prm->delay--;
				else
					return TRUE;
			} else if (speed_down_flag == TRUE) {
				speed_down_flag = FALSE;
				if (prm->delay < 50)
					prm->delay++;
				else
					return TRUE;
			}
			g_timeout_add_full(G_PRIORITY_DEFAULT, prm->delay, shift2left_callback, NULL, NULL);
			return FALSE;
		} else
			return TRUE;
	} else {
		/*
		 * When cairo_image surface scrolling completed or if request to
		 * reload or recompute everything, will create new surface or next one.
		 */
		env->suspend_rq = TRUE;
		if (env->c_surf != NULL) {
			cairo_surface_destroy(env->c_surf);
			env->c_surf = NULL;
		}
		i = OK;
		if (env->stream_fully_read || env->reload_rq) {
    			if (env->selection_mode == SINGLE && env->reload_rq)
				load_resource(resrc, NULL);
			else {
				while ((i = load_resource(resrc, feed_selection)) != OK) {
					if (i == CONNECT_TOO_MANY_ERRORS && env->selection_mode == MULTIPLE) {
						if (get_params()->disable_popups == 'n') {
							snprintf(tmp, 256,
							"\nFailed to connect %d times in a row.\n\n"
							"Please check your internet connection and/or "
							"your connection settings.\n\n"
							"Switch to single selection mode ?\n",
							CONNECT_FAIL_MAX);
							if (question_win(tmp) == YES) {
								env->selection_mode = SINGLE;
								break;
							}
						} else {
							fprintf(STD_ERR,
							"Failed to connect %d times in a row\n"
							"Please check your internet connection and/or "
							"your connection settings\n",
							CONNECT_FAIL_MAX);
							if (gtk_events_pending()) {
								if (gtk_main_iteration_do(FALSE))
									break;
							} else
#ifndef G_OS_WIN32
								sleep(CONNECT_FAIL_TIMEOUT);
#else
								Sleep(CONNECT_FAIL_TIMEOUT * 1000);
#endif
						}
					}
				}
			}
			check_time_load_resource(TIME_RESET);
		}
		compute_surface_and_win();
		shift_counter = prm->shift_size;	/* To avoid displaying twice the same thing */
		g_timeout_add_full(G_PRIORITY_DEFAULT, prm->delay, shift2left_callback, NULL, NULL);
		env->reload_rq = FALSE;
		env->compute_rq = FALSE;
		env->suspend_rq = FALSE;
		return FALSE;
	}
}

static gint display_time2()
{
	display_time(prm);
	return TRUE;
}

static gint get_win_position()	/* Only if win is 'draggable' */
{
	if (prm->windec == 'y')
		gtk_window_get_position(GTK_WINDOW(env->win), &prm->win_x, &prm->win_y);
	return TRUE;
}

/*
 * Reload stuff every <resrc->rss_ttl * 60> seconds
 */
static void check_time_load_resource(check_time_mode mode)
{
	static unsigned long	elapsed_time_in_sec;

	if (mode == TIME_RESET)
		elapsed_time_in_sec = 0;
	else if (mode == TIME_CHECK) {
		if (resrc->type == RESRC_FILE)
			resrc->rss_ttl = prm->rss_refresh;
		if ((++elapsed_time_in_sec) / 60 >= (unsigned long)resrc->rss_ttl) {
			elapsed_time_in_sec = 0;
			env->reload_rq = TRUE;
		}
	}
}

static gint check_time_load_resource2()
{
	check_time_load_resource(TIME_CHECK);
	return TRUE;
}

static void quit_with_cli_info(tickr_error_code error_code)
{
	if (resrc->fp != NULL)
		fclose(resrc->fp);
	xmlCleanupParser();
	free2(env);
	free2(resrc);
	free2(prm);
	fprintf(STD_ERR, "Try '"APP_CMD" --help' for more information\n");
	exit(error_code);
}

static void print_help()
{
	int	i;

	for (i = 0; get_help_str1()[i] != NULL; i++)
		fprintf(STD_OUT, "%s", get_help_str1()[i]);
	fprintf(STD_OUT, "\n");
}

static void print_version()
{
	fprintf(STD_OUT, APP_NAME" version "APP_VERSION_NUMBER"\n");
}

static void print_license()
{
	int	i;

	for (i = 0; get_license_str1()[i] != NULL; i++)
		fprintf(STD_OUT, "%s", get_license_str1()[i]);
	fprintf(STD_OUT, "%s\n", get_license_str2());
}

/* Reset display params as soon as scrolling has started. */
static gint update_everything()
{
	if (shift_counter > 4) {
		update_pixmap_from_opened_stream();
		return FALSE;
	} else
		return TRUE;
}

/* Reset opened stream to begining and recompute pixmap, in order to
 * update all display params.
 * (Hmmm... we should find a more informative name.)
 */
void update_pixmap_from_opened_stream()
{
	gboolean	suspend_rq_bak = env->suspend_rq;

	env->suspend_rq = TRUE;
	fseek(resrc->fp, 0, SEEK_SET);
	env->stream_fully_read = FALSE;
	env->compute_rq = TRUE;
	env->suspend_rq = suspend_rq_bak;
}

#ifdef G_OS_WIN32
/* This is meant to be used with the new GTK/GLib-win32 runtime. */
static int init_win32_mmtimer()
{
	TIMECAPS	tc;
	MMRESULT	mmr;
	unsigned	int highest_res;

	mmr = timeGetDevCaps(&tc, sizeof(TIMECAPS));
	highest_res = tc.wPeriodMin;
	if (timeBeginPeriod(highest_res) == TIMERR_NOERROR)
		return 0;
	else {
		fprintf(STD_ERR, "init_win32_mmtimer() error");
		return -1;
	}
}

static void create_win32_logfiles()
{
	time_t	time2 = time(NULL);

	STD_OUT = open_new_datafile_with_name(STDOUT_FILENAME1, "wb");
	STD_ERR = open_new_datafile_with_name(STDERR_FILENAME1, "wb");
	fprintf(STD_OUT, "%s", ctime(&time2));
	fprintf(STD_ERR, "%s", ctime(&time2));
}

/* Swap win32 log files every hour to prevent generating huge ones. */
static gint swap_win32_logfiles()
{
	static unsigned long	elapsed_time_in_sec = 0;
	static int		counter = 0;
	int			suspend_rq_bak;
	time_t			time2 = time(NULL);

	if ((++elapsed_time_in_sec) >= (unsigned long)3600) {
		elapsed_time_in_sec = 0;
		counter++;
		counter &= 1;
		suspend_rq_bak = env->suspend_rq;
		env->suspend_rq = TRUE;
		if (STD_OUT != NULL) {
			if (fclose(STD_OUT) != 0)
				big_error(WIN32V_ERROR, 2, "Can't close STD_OUT: ", strerror(errno));
			STD_OUT = NULL;
		}
		if (STD_ERR != NULL) {
			if (fclose(STD_ERR) != 0)
				big_error(WIN32V_ERROR, 2, "Can't close STD_ERR: ", strerror(errno));
			STD_ERR = NULL;
		}
		if (counter == 0) {
			STD_OUT = open_new_datafile_with_name(STDOUT_FILENAME1, "wb");
			STD_ERR = open_new_datafile_with_name(STDERR_FILENAME1, "wb");
		} else {
			STD_OUT = open_new_datafile_with_name(STDOUT_FILENAME2, "wb");
			STD_ERR = open_new_datafile_with_name(STDERR_FILENAME2, "wb");
		}
		fprintf(STD_OUT, "%s", ctime(&time2));
		fprintf(STD_ERR, "%s", ctime(&time2));
		env->suspend_rq = suspend_rq_bak;
	}
	return TRUE;
}
#endif

/* TODO: is this still needed ? */
/* Question at program start-up about new feed list format conversion:
 * if version >= 0.6.2 and feed list exists and feed list backup doesn't
 * exist, create backup and convert to new format. */
#define URL_LIST_BAK_FILE	URL_LIST_FILE"-bak"
static void test_convert_local_feed_list()
{
	FILE	*fp1, *fp2;
	char	*str;
	size_t	str_size = 8 * 1024;

	if (strncmp(APP_VERSION_NUMBER, "0.6.2", 5) >= 0)
		if (g_file_test(get_datafile_full_name_from_name(URL_LIST_FILE), G_FILE_TEST_EXISTS) &&\
				!g_file_test(get_datafile_full_name_from_name(URL_LIST_BAK_FILE), G_FILE_TEST_EXISTS))
			if (question_win(APP_NAME" version 0.6.2 or later uses a new feed list format internally.\n"
					"Do you wish your local feed list to be checked and eventually converted ?\n"
					"(Doing this ONCE is strongly RECOMMENDED.)") == YES) {
				fp1 = open_new_datafile_with_name(URL_LIST_FILE, "rb");
				fp2 = open_new_datafile_with_name(URL_LIST_BAK_FILE, "wb");
				str = malloc2(str_size * sizeof(char));
#ifndef G_OS_WIN32
				while (getline(&str, &str_size, fp1) != -1)
#else
				while (fgets(str, str_size, fp1) != NULL)
#endif
					fprintf(fp2, "%s", str);
				fclose(fp1);
				fclose(fp2);
				fp1 = open_new_datafile_with_name(URL_LIST_FILE, "wb");
				fp2 = open_new_datafile_with_name(URL_LIST_BAK_FILE, "rb");
#ifndef G_OS_WIN32
				while (getline(&str, &str_size, fp2) != -1)
#else
				while (fgets(str, str_size, fp2) != NULL)
#endif
					if (strncmp(str + 1, "http", 4) == 0 || strncmp(str + 1, "file", 4) == 0)
						fprintf(fp1, "%c   %s", str[0], str + 1);
					else
						fprintf(fp1, "%s", str);
				free2(str);
				fclose(fp1);
				fclose(fp2);
			}
}

int main(int argc, char *argv[])
{
	GdkPixbuf	*pixb = NULL;
	GtkIconFactory	*icon_factory = NULL;
	GError		*error = NULL;
#ifndef G_OS_WIN32
	GdkColormap	*colormap = NULL;
#endif
	int		n_options, n_resources, i, j;
#ifdef G_OS_WIN32
	WSADATA		wsadata;
#endif

	gtk_init(&argc, &argv);
	LIBXML_TEST_VERSION
#ifndef LIBXML_TREE_ENABLED
# error Libxml2: tree support not compiled in
#endif
	xmlInitParser();
	env = malloc2(sizeof(TickerEnv));
	resrc = malloc2(sizeof(Resource));
	prm = malloc2(sizeof(Params));
	env->win = env->drw_a = env->drwa_clock = NULL;
	env->c_surf = NULL,
	env->screen = NULL;
	env->visual = NULL;
	env->screen_w = env->screen_h = env->depth = 0;
	env->drwa_width = env->drwa_height = env->surf_width = env->surf_height = 0;
	env->active_link[0] = '\0';
	env->mouse_x_in_drwa = -1;
	env->suspend_rq = env->compute_rq = env->reload_rq = env->stream_fully_read = FALSE;
	/* env->selection_mode: default = MULTIPLE, set to SINGLE internally if
	 * selection is unavalaible/empty or after picking a single feed. */
	env->selection_mode = MULTIPLE;
	/*
	 * Toplevel window
	 */
	env->win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(env->win), APP_NAME"-"APP_VERSION_NUMBER);
	gtk_container_set_border_width(GTK_CONTAINER(env->win), 0);
	g_signal_connect(G_OBJECT(env->win), "delete_event", G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect(G_OBJECT(env->win), "destroy", G_CALLBACK(gtk_widget_destroy), &(env->win));

#ifdef G_OS_WIN32
	if ((i = WSAStartup(MAKEWORD(2, 2), &wsadata)) != 0)
		big_error(WIN32V_ERROR, 2, "WSAStartup() error ", itoa2(i));
	else if (LOBYTE(wsadata.wVersion) != 2 || HIBYTE(wsadata.wVersion) != 2) {
		WSACleanup();
		big_error(WIN32V_ERROR, 1, "Couldn't find a usable version of Winsock.dll");
	} else if (get_progfiles_dir() == NULL)
		big_error(WIN32V_ERROR, 1, "Can't find Program Files directory");
	else if (get_appdata_dir() == NULL)
		big_error(WIN32V_ERROR, 1, "Can't find Application Data directory");
	else if (init_win32_mmtimer() != 0)
		if (question_win("Can't initialize win32 multimedia timer. "
				"Scrolling may be very slow. Continue ?") == NO)
			exit(WIN32V_ERROR);
#endif
	/*
	 * Create dir in user home dir if it doesn't exist.
	 */
	g_mkdir(get_datadir_full_path(), S_IRWXU);
#ifdef G_OS_WIN32
	create_win32_logfiles();
#endif
	/*
	 * Get help, show license, set instance id, run with no-ui, or check available system fonts.
	 */
	if (argc >= 2 && (strcmp(argv[1], "-help") == 0 || strcmp(argv[1], "--help") == 0 ||\
			strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "-?") == 0)) {
		print_help();
		return 0;
	} else if (argc >= 2 && (strcmp(argv[1], "-version") == 0 || strcmp(argv[1], "--version") == 0 ||\
			strcmp(argv[1], "-v") == 0)) {
		print_version();
		return 0;
	} else if (argc >= 2 && (strcmp(argv[1], "-license") == 0 || strcmp(argv[1], "--license") == 0 ||\
			strcmp(argv[1], "-l") == 0)) {
		print_license();
		return 0;
	} else if (argc >= 2 && strncmp(argv[1], "-instance-id=", strlen("-instance-id=")) == 0) {
		if ((instance_id = atoi(argv[1] + strlen("-instance-id="))) < 1 || instance_id > 99) {
			fprintf(STD_ERR, "Instance ID invalid value: %d\n", instance_id);
			instance_id = 0;
		}
	} else if (argc >= 2 && strncmp(argv[1], "-no-ui", strlen("-no-ui")) == 0)
		no_ui = TRUE;
	else if (argc >= 2 && (strcmp(argv[1], "-dumpfontlist") == 0 || strcmp(argv[1], "--dumpfontlist") == 0)) {
		dump_font_list();
		return 0;
	}
	set_tickr_icon_to_dialog(GTK_WINDOW(env->win));
	/*
	 * Create rss icon
	 */
	pixb = gdk_pixbuf_new_from_file(get_imagefile_full_name_from_name(RSS_ICON), &error);
	icon_factory = gtk_icon_factory_new();
	gtk_icon_factory_add(icon_factory, RSS_ICON, gtk_icon_set_new_from_pixbuf(pixb));
	gtk_icon_factory_add_default(icon_factory);
	if (pixb != NULL)
		g_object_unref(pixb);
	/*
	 * Get some system graphical params
	 */
	env->screen = gtk_widget_get_screen(env->win);
#ifndef G_OS_WIN32
	if ((colormap = gdk_screen_get_rgba_colormap(env->screen)) != NULL) {
#ifdef VERBOSE_OUTPUT
		fprintf(STD_OUT, "Composited GDK screen found\n");
#endif
	} else
		colormap = gdk_screen_get_rgb_colormap(env->screen);
	gtk_widget_set_colormap(env->win, colormap);
	env->visual = gdk_colormap_get_visual(colormap);
#else
	env->visual = gdk_visual_get_best();
#endif
	env->depth = env->visual->depth;
	env->screen_w = gdk_screen_get_width(gtk_window_get_screen(GTK_WINDOW(env->win)));
	env->screen_h = gdk_screen_get_height(gtk_window_get_screen(GTK_WINDOW(env->win)));
#ifdef VERBOSE_OUTPUT
	fprintf(STD_OUT, "GDK visual: screen width = %d, screen height = %d, depth = %d\n",
		env->screen_w, env->screen_h, env->depth);
	if (no_ui)
		warning(FALSE, 2, APP_NAME" will run with the 'no-ui' option.\n",
			"Opening of UI elements which can modify settings and/or \n"
			"URL list/selection is disabled.");
#endif
	resrc->type = RESRC_UNSPECIFIED;
	resrc->format = RSS_FORMAT_UNDETERMINED;
	resrc->id[0] = '\0';
	resrc->xml_dump[0] = '\0';
	resrc->fp = NULL;
	resrc->fp_extra = NULL;
	for (i = 0; i < NFEEDLINKANDOFFSETMAX; i++) {
		resrc->link_and_offset[i].url[0] = '\0';
		resrc->link_and_offset[i].offset_in_surface = 0;
	}
	init_authentication();
	init_proxy();
	/*
	 * Options and resource stuff
	 */
	set_default_options(prm);	/* init all params */
	if (g_file_test(get_datafile_full_name_from_name(CONFIG_FILE), G_FILE_TEST_EXISTS))
		get_config_file_options(prm);
	else
		quick_setup(prm);

/* NEW and experimental/unstable */
#ifdef DOUBLE_LINE_TICKER
	prm->feed_title = 'n';
	prm->item_title = 'n';
	prm->item_description = 'y';
	prm->clock = 'n';
#endif
	n_options = 0;
	n_resources = 0;
	if (argc > 1) {
		/* Parse args: first -options... then resource - the rest is ignored. */
		for (i = 1; i < argc; i++)
			if (argv[i][0] == '-')
				n_options ++;
			else
				break;
		if (i < argc)
			n_resources++;
		if (n_resources == 0) {
			fprintf(STD_ERR, "No resource specified\n");
			if (RESOURCE_NONE_QUIT)
				quit_with_cli_info(RESOURCE_NONE);
		} else {
			env->selection_mode = SINGLE;
			/* will open text file or get rss feed */
			str_n_cpy(resrc->id, argv[n_options + 1], FILE_NAME_MAXLEN);
		}
		if (n_options > 0)
			if (parse_options_array(prm, n_options, (const char **)(argv + 1)) != OK)
				if (OPTION_ERROR_QUIT)
					quit_with_cli_info(OPTION_ERROR);
	} else {
		fprintf(STD_ERR, "No resource specified\n");
		if (RESOURCE_NONE_QUIT)
			quit_with_cli_info(RESOURCE_NONE);
	}
	compute_auth_and_proxy_str();
	/*
	 * Feed list stuff
	 */
	test_convert_local_feed_list();
	if ((i = f_list_load_from_file(&feed_list, NULL)) == OK) {
		feed_list = f_list_sort(feed_list);
		feed_list = f_list_first(feed_list);
		f_list_save_to_file(feed_list, NULL);
	}
	j = build_feed_selection_from_feed_list();
	if (n_resources == 0 && env->selection_mode == MULTIPLE && i == OK && j == OK)
		fprintf(STD_OUT, "Will use feed selection\n");
	else {
		if (env->selection_mode == MULTIPLE && (i != OK || j != OK)) {
			if (j == SELECTION_EMPTY)
				warning(TRUE, 1, "No feed selected - "
					"Switching to single selection mode");
			else if (j == SELECTION_ERROR)
				warning(TRUE, 1, "No feed selection available - "
					"Switching to single selection mode");
		}
		env->selection_mode = SINGLE;
		if (n_resources == 0) {
			if (prm->homefeed[0] != '\0') {
				str_n_cpy(resrc->id, prm->homefeed, FILE_NAME_MAXLEN);
				fprintf(STD_OUT, "Will use default resource (Homefeed): %s\n", resrc->id);
			} else {
				fprintf(STD_ERR, "No default resource available\n");
				if (RESOURCE_NONE_QUIT)
					quit_with_cli_info(RESOURCE_NONE);
			}
		}
	}
	first_feed();
	if (load_resource(resrc, feed_selection) != OK)
		if (RESOURCE_NONE_QUIT)
			quit_with_cli_info(RESOURCE_NONE);
	resrc->rss_ttl = prm->rss_refresh;
	first_feed();
	/*
	 * Set window layout
	 */
	main_hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(env->win), main_hbox);
	popup_menu = get_popup_menu_menu(NULL);
	popup_menu_accel_group_attached = FALSE;
	/*
	 * Create 2 drawing areas
	 */
	env->drw_a = gtk_drawing_area_new();
	/* We disable double buffering for drawing area and will use
	 * gdk_window_begin_paint_rect() and gdk_window_end_paint().
	 */
	gtk_widget_set_double_buffered(env->drw_a, FALSE);
	env->drwa_clock = gtk_drawing_area_new();
	gtk_widget_add_events(env->drw_a, GDK_ALL_EVENTS_MASK);	/* change mask? */

	g_signal_connect(G_OBJECT(env->drw_a), "button-press-event", G_CALLBACK(left_click_on_drawing_area), NULL);
	g_signal_connect(G_OBJECT(env->drw_a), "button-press-event", G_CALLBACK(right_click_on_drawing_area), NULL);
	g_signal_connect(G_OBJECT(env->drw_a), "enter-notify-event", G_CALLBACK(mouse_over_drawing_area), NULL);
	g_signal_connect(G_OBJECT(env->drw_a), "leave-notify-event", G_CALLBACK(mouse_over_drawing_area), NULL);
	g_signal_connect(G_OBJECT(popup_menu), "expose-event", G_CALLBACK(mouse_over_opened_popup_menu), NULL);
	g_signal_connect(G_OBJECT(popup_menu), "leave-notify-event", G_CALLBACK(mouse_over_opened_popup_menu), NULL);
	g_signal_connect(G_OBJECT(env->drw_a), "scroll-event", G_CALLBACK(mouse_wheel_scroll_on_drawing_area), NULL);
	g_signal_connect(G_OBJECT(env->drw_a), "motion-notify-event", G_CALLBACK(track_mouse_position_x), NULL);

	vbox_ticker = gtk_vbox_new(FALSE, 0);	/* vbox_ticker is only used to have a v-centered ticker */
	gtk_box_pack_start(GTK_BOX(vbox_ticker), GTK_WIDGET(env->drw_a), TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(main_hbox), vbox_ticker, FALSE, FALSE, 0);
	vbox_clock = gtk_vbox_new(FALSE, 0);	/* vbox_clock is only used to have a v-centered clock */
	gtk_box_pack_start(GTK_BOX(vbox_clock), GTK_WIDGET(env->drwa_clock), TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(main_hbox), vbox_clock, FALSE, FALSE, 0);
	/*
	 * Create cairo image surface from text file or rss feed and compute stuff to draw window.
	 */
	compute_surface_and_win();

	/* === Timeout callbacks === */
	/*
	 * Refresh the image every <delay> milliseconds
	 */
	g_timeout_add_full(G_PRIORITY_DEFAULT, prm->delay, shift2left_callback, NULL, NULL);
	/*
	 * Display clock every 0.5 seconds
	 */
	g_timeout_add_full(G_PRIORITY_DEFAULT, 500, display_time2, NULL, NULL);
	/*
	 * Get current win position (if 'dragged') every 0.5 seconds
	 */
	g_timeout_add_full(G_PRIORITY_DEFAULT, 500, get_win_position, NULL, NULL);
	/*
	 * Check every sec until elapsed time in mn > <resrc->rss_ttl> mn then reload rss feed (or text file)
	 */
	check_time_load_resource(TIME_RESET);
	g_timeout_add_full(G_PRIORITY_DEFAULT, 1000, check_time_load_resource2, NULL, NULL);

	g_timeout_add_full(G_PRIORITY_DEFAULT, 500, update_win_dims, NULL, NULL);

	g_timeout_add_full(G_PRIORITY_DEFAULT, 100, update_everything, NULL, NULL);
	/*
	 * Check every sec until elapsed time >= 1 hour then swap win32 log files to prevent generating huge ones
	 */
#ifdef G_OS_WIN32
	g_timeout_add_full(G_PRIORITY_DEFAULT, 1000, swap_win32_logfiles, NULL, NULL);
#endif

	env->suspend_rq = FALSE;
	env->compute_rq = FALSE;
	env->reload_rq = TRUE;

	gtk_widget_show_all(env->win);
	update_win_dims();
	gtk_main();
	/*
	 * Cleanup stuff
	 */
	if (resrc->fp != NULL)
		fclose(resrc->fp);
	xmlCleanupParser();
#ifdef G_OS_WIN32
	WSACleanup();
#endif
	free2(env);
	free2(resrc);
	free2(prm);
	return 0;
}
